#include <config.h>
#include "lisp.h"
#include <unistd.h>
#include <fcntl.h>

extern char __data_start__; /* start of data segment */
extern char __data_end__; /* end of data segment */

/* structure to write into first block of map file */
struct map_data
{
    char* start;			/* start of data area */
    size_t length;			/* length of data area */
};

extern int use_heap;	/* defined in bemalloc.c */
extern int allocated = 0;
extern int freed = 0;

/* maps in the data and alloc area from the map file */
int
mapin_data (char* name)
{
    struct map_data map_data;
    int infd;

    /* open map file */
    infd = open(name, O_RDONLY);
    if (infd < 0) {
        printf("Map file not available, running bare Emacs...\n");
        return 0;
    }

    /* read the header data */
    if (read(infd, &map_data, sizeof(map_data)) < 0) {
        printf("Error reading header\n");
        return 0;
    }

    /* check some stuff */
    if (map_data.start != &__data_start__) {
        printf ("Start of data area has moved: cannot map in data.\n");
        return 0;
    }

    /* all's well, read in the data */
    if (read(infd, map_data.start, map_data.length) < 0) {
        printf("Error reading map_data data.\n");
        return 0;
    }

    return 1;
}

/* writes the data and alloc area to the map file */
int
mapout_data (char* into)
{
    struct map_data map_data;
    int outfd;

    map_data.start = &__data_start__;
    map_data.length = &__data_end__ - map_data.start;

    /* printf("Writing out map file to: %s\n", into);
       printf("Allocated: %d\nFreed: %d\n", allocated, freed); */

    if (use_heap) {
        error("Out of initial allocation. Must rebuild emacs with more "
              "memory (BE_ALLOCATION_SIZE).");
        return 0;
    }

    if ((outfd = open(into , O_RDWR|O_CREAT)) < 0) {
        printf("Can't create map file [%s]\n", into);
        return 0;
    }

    if (write(outfd, &map_data, sizeof(map_data)) < 0) {
        printf("Error writing map_data header.\n");
        return 0;
    }

    if (write(outfd, map_data.start, map_data.length) < 0) {
        printf("Error writing map_data data.\n");
        return 0;
    }

    return 1;
}
